
/* 
   This is a GAI example applet written by Jonas Aaberg <cja@gmx.net> 2003.

   It displays the current time. 

   Released under GNU GPL.

*/

#include <time.h>
#include <stdio.h>
#include <gai/gai.h>
#include "config.h"


static unsigned char r,g,b, a=0xff;
static int show_sec, ampm;

static GaiColourSelector font_colour = {"Font colour: ", &r, &g, &b, &a, 1};
static GaiCheckButton sec_check = {"Show seconds", &show_sec, &show_sec};
static GaiCheckButton ampm_check = {"12 Hour format", &ampm, &ampm};

static GaiBox main_box[] = {{GAI_COLOURSELECTOR,	&font_colour},
			    {GAI_CHECKBUTTON,		&sec_check},
			    {GAI_CHECKBUTTON,		&ampm_check},
			    {GAI_END,			NULL}};

static GaiNoteBook gai_clock_pref[] = {{"gai-clock", 	(void *)&main_box},
				       {GAI_END,	NULL}};


char *create_clock_str(void)
{
    time_t my_t;
    struct tm *my_tm;
    my_t = time(NULL);
    my_tm = localtime(&my_t);

    if(!ampm){
	if(show_sec)
	    return g_strdup_printf("%d:%.2d:%.2d",my_tm->tm_hour, my_tm->tm_min, my_tm->tm_sec);
	else
	    return g_strdup_printf("%d:%.2d",my_tm->tm_hour, my_tm->tm_min);
    } else {
	if(show_sec){
	    if(my_tm->tm_hour >12)
		return g_strdup_printf("%d:%.2d:%.2d PM",my_tm->tm_hour%12, my_tm->tm_min, my_tm->tm_sec);
	    else
		return g_strdup_printf("%d:%.2d:%.2d AM",my_tm->tm_hour, my_tm->tm_min, my_tm->tm_sec);
	} else {
	    if(my_tm->tm_hour >12)
		return g_strdup_printf("%d:%.2d PM",my_tm->tm_hour%12, my_tm->tm_min);
	    else
		return g_strdup_printf("%d:%.2d AM",my_tm->tm_hour, my_tm->tm_min);
	}
    }

}

void update_clock(void)
{

    char *tmp;
    GdkPixbuf *clk_image;

    tmp = create_clock_str();
    clk_image = gai_text_create(tmp, "arial", 12, GAI_TEXT_NORMAL, r,g,b);
    g_free(tmp);

    gai_draw_bg(clk_image,0,0,
	     gdk_pixbuf_get_width(clk_image),
	     gdk_pixbuf_get_height(clk_image),
	     0,0);

    gai_draw_update_bg();
    g_object_unref(clk_image);

}

void gai_clock_settings(void)
{
    char *tmp;
    GdkPixbuf *clk_image;

    gai_save_int("gai-clock/red",(int)r);
    gai_save_int("gai-clock/green",(int)g);
    gai_save_int("gai-clock/blue",(int)b);

    gai_save_int("gai-clock/show_sec", show_sec);
    gai_save_int("gai-clock/ampm", ampm);

    tmp = create_clock_str();
    clk_image = gai_text_create(tmp, "arial", 12, GAI_TEXT_NORMAL, r,g,b);
    g_free(tmp);

    gai_background_set(gdk_pixbuf_get_width(clk_image), 
		       gdk_pixbuf_get_height(clk_image), 
		       gdk_pixbuf_get_height(clk_image), FALSE);

    g_object_unref(clk_image);

}

int main(int argc, char **argv)
{
    GdkPixbuf *clk_image;
    GaiFlagsType gf = GAI_FLAGS_NEVER_ROTATE;
    char *tmp;

    gai_init2(&applet_defines, &argc, &argv);

    r = (unsigned char)gai_load_int_with_default("gai-clock/red",0x00);
    g = (unsigned char)gai_load_int_with_default("gai-clock/green",0x00);
    b = (unsigned char)gai_load_int_with_default("gai-clock/blue",0x00);

    show_sec = gai_load_int_with_default("gai-clock/show_sec", 1);
    ampm = gai_load_int_with_default("gai-clock/ampm", 0);

    tmp = create_clock_str();
    clk_image = gai_text_create(tmp, "arial", 11, GAI_TEXT_NORMAL, r,g,b);
    g_free(tmp);

    gai_background_set(gdk_pixbuf_get_width(clk_image), 
		       gdk_pixbuf_get_height(clk_image), 
		       gdk_pixbuf_get_height(clk_image), FALSE);

    gai_flags_set(gf);

    g_object_unref(clk_image);


    gai_preferences("GAI Digital Clock - Preferences",
		    gai_clock_pref,
		    "Select if you want the seconds to be shown or not,\n"
		    "and if you want the time in AM/PM format or in 24 hour format\n",
		    (GaiCallback0 *)gai_clock_settings,
		    NULL);

    /* Update two times a second */
    gai_signal_on_update((GaiCallback0 *)update_clock, 500, NULL);

    gai_start();

    return 0;
}
